@@ -29,7 +29,7 @@ manager protocol), the modifications are only applied when the context
 exits for the last time.
 
 For tokenized documents (see :class:`parce.Document`), parce inherits from this
-base class (see the :mod:`~parce.treedocument` module).
+base class (see the :mod:`~parce.work` module).
 
 You can use a Cursor to keep track of positions in a document. The position
 (and selection) of a Cursor is adjusted when the text in the document is
@@ -204,8 +204,12 @@ class AbstractDocument(mutablestring.AbstractMutableString):
         repl = lambda m: mapping[m.group()]
         self.re_sub(expr, repl, start, end, count)
 
-    def contents_changed(self, position, removed, added):
-        """Called by _apply(). The default implementation does nothing."""
+    def text_changed(self, position, removed, added):
+        """Called after ``_update_text()``.
+
+        The default implementation does nothing.
+
+        """
         pass
 
 
@@ -219,11 +223,11 @@ class Document(AbstractDocument, mutablestring.MutableString, util.Observable):
     It also inherits from :class:`~parce.util.Observable` and emits the
     following events:
 
-    ``"contents_change" (position, removed, added)``:
+    ``"text_change" (position, removed, added)``:
         emitted with ``position``, ``removed``, ``added`` arguments whenever the
         text changes
 
-    ``"contents_changed"``:
+    ``"text_changed"``:
         emitted directly afther the previous event, but without arguments
 
     ``"modification_changed" (bool)``:
@@ -234,7 +238,7 @@ class Document(AbstractDocument, mutablestring.MutableString, util.Observable):
         emitted when the availability of :meth:`undo` changes
 
     ``"redo_available" (bool)``:
-        emitted when the availability of :meth:`redo` changes
+        emitted when the availability of :meth:`redo` changes.
 
     """
     _in_undo = util.Switch()
@@ -267,15 +271,26 @@ class Document(AbstractDocument, mutablestring.MutableString, util.Observable):
         """Apply the changes to the text."""
         with self._check_undo_state():
             if self.undo_redo_enabled:
-                self._store_undo()
+                self._store_undo(self._reverse_changes(changes))
             AbstractDocument._update_text(self, changes)
             mutablestring.MutableString._update_text(self, changes)
         if not (self._in_undo or self._in_redo):
             self.set_modified(True) # othw this is handled by undo/redo
 
-    def _store_undo(self):
+    def _reverse_changes(self, changes):
+        """Return the changes that would be needed to undo the given list of changes."""
+        def reverse_changes():
+            head = 0
+            current_text = self.text()
+            for start, end, text in changes:
+                head += start
+                yield (head, head + len(text), current_text[start:end])
+                head += len(text) - end
+        return list(reverse_changes())
+
+    def _store_undo(self, changes):
         """Store changes needed to reconstruct the previous state."""
-        state = [self._get_reverse_changes(), self.modified()]
+        state = [changes, self.modified()]
         if self._in_undo:
             self._redo_stack.append(state)
         else:
@@ -283,13 +298,15 @@ class Document(AbstractDocument, mutablestring.MutableString, util.Observable):
             if not self._in_redo:
                 self._redo_stack.clear()
 
-    def _apply_undo(self, stack):
+    def _apply_undo_redo(self, stack):
         """Apply changes from the specified stack (undo or redo)."""
         if self._edit_context > 0:
-            raise RuntimeError("can't redo or redo while in edit context")
+            raise RuntimeError("can't undo or redo while in edit context")
         if stack:
-            self._changes, modified = stack.pop()
-            self._apply_changes()
+            changes, modified = stack.pop()
+            with self:
+                for start, end, text in changes:
+                    self[start:end] = text
             self.set_modified(modified)
 
     @contextlib.contextmanager
@@ -319,12 +336,12 @@ class Document(AbstractDocument, mutablestring.MutableString, util.Observable):
     def undo(self):
         """Undo the last modification."""
         with self._in_undo:
-            self._apply_undo(self._undo_stack)
+            self._apply_undo_redo(self._undo_stack)
 
     def redo(self):
         """Redo the last undone modification."""
         with self._in_redo:
-            self._apply_undo(self._redo_stack)
+            self._apply_undo_redo(self._redo_stack)
 
     def clear_undo_redo(self):
         """Clear the undo/redo stack."""
@@ -340,15 +357,15 @@ class Document(AbstractDocument, mutablestring.MutableString, util.Observable):
         """Return True if redo is possible."""
         return bool(self._redo_stack)
 
-    def contents_changed(self, position, removed, added):
-        """Called by ``_apply_changes()``.
+    def text_changed(self, position, removed, added):
+        """Called after ``_update_text()`` has been called.
 
-        The default implementation emits the ``"contents_change"`` and
-        ``"contents_changed"`` events.
+        The default implementation emits the ``"text_change"`` and
+        ``"text_changed"`` events.
 
         """
-        self.emit("contents_change", position, removed, added)
-        self.emit("contents_changed")
+        self.emit("text_change", position, removed, added)
+        self.emit("text_changed")
 
 
 class AbstractTextRange:
@@ -417,15 +434,22 @@ class AbstractTextRange:
         return self.pos <= other.pos
 
     def token(self):
-        """Convenience method returning the :class:`~parce.tree.Token` at our pos."""
+        """Convenience method returning the :class:`~parce.tree.Token` at our pos.
+
+        The Document must have the :class:`~.work.WorkerDocumentMixin` class
+        mixed in (i.e. have the :meth:`~.work.WorkerDocumentMixin.token`
+        method.
+
+        """
         return self.document().token(self.pos)
 
     def tokens(self):
         """Convenience method yielding all Tokens that are in or overlap this
         text range.
 
-        The Document must have the :class:`~parce.treedocument.TreeDocument`
-        class mixed in (i.e. have the ``get_root()`` method.
+        The Document must have the :class:`~.work.WorkerDocumentMixin` class
+        mixed in (i.e. have the :meth:`~.work.WorkerDocumentMixin.get_root`
+        method.
 
         """
         return self.document().get_root(True).tokens_range(self.pos, self.end)
@@ -442,19 +466,27 @@ class Cursor(AbstractTextRange):
     remains the same. But when text is inserted at the end of a cursor, the
     ``end`` position (if not None) moves along with the new text. E.g.::
 
-        d = Document('hi there, folks!')
-        c = Cursor(d, 8, 8)
-        with d:
-            d[8:8] = 'new text'
-        c.pos, c.end --> (8, 16)
+        >>> from parce.document import Document, Cursor
+        >>> d = Document('hi there, folks!')
+        >>> c = Cursor(d, 8, 8)
+        >>> with d:
+        ...     d[8:8] = 'new text'
+        ...
+        >>> c.pos, c.end
+        (8, 16)
 
     You can also use a Cursor as key while editing a document::
 
-        c = Cursor(d, 8, 8)
-        with d:
-            d[c] = 'new text'
+        >>> c = Cursor(d, 8, 8)
+        >>> with d:
+        ...     d[c] = 'new text'
 
-    You cannot alter the document via the Cursor.
+    You cannot alter the document via the Cursor. All move and select methods
+    return the cursor again, so they can be chained::
+
+        >>> c = Cursor(d).select_all()
+        >>> c.pos, c.end
+        (0, None)
 
     """
     __slots__ = ("__weakref__",)
@@ -472,19 +504,32 @@ class Cursor(AbstractTextRange):
         """Yield the Blocks from pos to end."""
         yield from self.document().blocks(self.pos, self.end)
 
+    def move_start_of_block(self):
+        """Move pos and end to the start of the current block. Returns self."""
+        self.pos = self.end = self.document().find_start_of_block(self.pos)
+        return self
+
+    def move_end_of_block(self):
+        """Move pos and end to the end of the current block. Returns self."""
+        self.pos = self.end = self.document().find_end_of_block(self.pos)
+        return self
+
     def select(self, pos, end=-1):
-        """Change pos and end in one go. End defaults to pos."""
+        """Change pos and end in one go. End defaults to pos. Returns self."""
         self.pos = pos
         self.end = pos if end == -1 else end
+        return self
 
     def select_all(self):
-        """Set pos to 0 and end to None; selecting all text."""
+        """Set pos to 0 and end to None; selecting all text. Returns self."""
         self.pos = 0
         self.end = None
+        return self
 
     def select_none(self):
-        """Set end to pos."""
+        """Set end to pos. Returns self."""
         self.end = self.pos
+        return self
 
     def has_selection(self):
         """Return True if text is selected."""
@@ -492,29 +537,42 @@ class Cursor(AbstractTextRange):
         return self.pos < end
 
     def select_start_of_block(self):
-        """Moves the selection pos to the beginning of the current line."""
+        """Moves the selection pos to the beginning of the current line.
+
+        Returns self.
+
+        """
         self.pos = self.document().find_start_of_block(self.pos)
+        return self
 
     def select_end_of_block(self):
-        """Moves the selection end (if not None) to the end of its line."""
+        """Moves the selection end (if not None) to the end of its line.
+
+        Returns self.
+
+        """
         if self.end is not None:
             self.end = self.document().find_end_of_block(self.end)
+        return self
 
     def lstrip(self, chars=None):
         """Move pos to the right, if specified characters can be skipped.
 
         By default whitespace is skipped, like Python's lstrip() string method.
+        Returns self.
 
         """
         text = self.text()
         if text:
             offset = len(text) - len(text.lstrip(chars))
             self.pos += offset
+        return self
 
     def rstrip(self, chars=None):
         """Move end to the left, if specified characters can be skipped.
 
         By default whitespace is skipped, like Python's rstrip() string method.
+        Returns self.
 
         """
         text = self.text()
@@ -525,12 +583,13 @@ class Cursor(AbstractTextRange):
                 if self.end is None or self.end > doc_length:
                     self.end = doc_length
                 self.end -= offset
+        return self
 
     def strip(self, chars=None):
-        """Adjust pos and end, like Python's strip() method."""
+        """Adjust pos and end, like Python's strip() method. Returns self."""
         self.rstrip(chars)
         self.lstrip(chars)
-
+        return self
 
 
 class Block(AbstractTextRange):
@@ -580,8 +639,9 @@ class Block(AbstractTextRange):
         """Convenience method returning a tuple with all Tokens that are in
         or overlap this block.
 
-        The Document must have the :class:`~parce.treedocument.TreeDocument`
-        class mixed in (i.e. have the ``get_root()`` method.
+        The Document must have the :class:`~.work.WorkerDocumentMixin` class
+        mixed in (i.e. have the :meth:`~.work.WorkerDocumentMixin.get_root`
+        method.
 
         """
         return tuple(super().tokens())
