@@ -89,6 +89,7 @@ music_commands = (
     'addChordShape', # since 2.16
     'addInstrumentDefinition',
     'addQuote',
+    'after',         # since 2.23
     'afterGrace',
     #'afterGraceFraction', # this is a parser variable
     'aikenHeads',
@@ -97,6 +98,7 @@ music_commands = (
     'alterBroken', # since 2.18 (?)
     'alternative',
     #'AncientRemoveEmptyStaffContext',
+    'ambitusAfter', # since 22.1
     'appendToTag', # since 2.16
     'applyContext',
     'applyMusic',
@@ -231,6 +233,7 @@ music_commands = (
     'inclinatum',
     'includePageLayoutFile',
     'indent',
+    'inStaffSegno', # since 2.16
     'instrumentSwitch',
     'instrumentTransposition',
     'interscoreline',
@@ -389,12 +392,14 @@ music_commands = (
     'tupletUp',
     'tweak',
     'unaCorda',
+    'unfolded',     # since 2.23
     'unfoldRepeats',
     'unHideNotes',
     'unit',
     'versus',
     'virga',
     'virgula',
+    'vshape',       # since 2.23
     'voiceFour',
     'voiceFourStyle',
     'voiceNeutralStyle',
@@ -404,6 +409,7 @@ music_commands = (
     'voiceThreeStyle',
     'voiceTwo',
     'voiceTwoStyle',
+    'volta',    # since 2.23
     'walkerHeads',
     'walkerHeadsMinor',
     'whiteTriangleMarkup',
@@ -551,6 +557,7 @@ markup_commands_nargs = (
     'fraction',
     'halign',
     'hcenter-in',
+    'if',       # since 2.23
     'lower',
     'magnify',
     'map-markup-commands',  # since 2.20
@@ -571,6 +578,7 @@ markup_commands_nargs = (
     'table',        # since 2.20
     'translate',
     'translate-scaled',
+    'unless',       # since 2.23
     'with-color',
     'with-dimensions-from', # since 2.20
     'with-link',
@@ -1803,166 +1811,212 @@ pitch_names = {
         "sikk": (6, DOUBLE_SHARP),
     },
 }
-pitch_names["español"] = pitch_names["espanol"]
-#pitch_names["francais"] = pitch_names["français"]
 
 # a set with all the pitch names for fast membership testing
 all_pitch_names = frozenset(name for d in pitch_names.values() for name in d)
 
+# some synonyms
+pitch_names["español"] = pitch_names["espanol"]
+pitch_names["francais"] = pitch_names["français"]
 
-drum_pitch_names = (
-    'acousticbassdrum',
-    'bassdrum',
-    'hisidestick',
-    'sidestick',
-    'losidestick',
-    'acousticsnare',
-    'snare',
-    'handclap',
-    'electricsnare',
-    'lowfloortom',
-    'closedhihat',
-    'hihat',
-    'highfloortom',
-    'pedalhihat',
-    'lowtom',
-    'openhihat',
-    'halfopenhihat',
-    'lowmidtom',
-    'himidtom',
-    'crashcymbala',
-    'crashcymbal',
-    'hightom',
-    'ridecymbala',
-    'ridecymbal',
-    'chinesecymbal',
-    'ridebell',
-    'tambourine',
-    'splashcymbal',
-    'cowbell',
-    'crashcymbalb',
-    'vibraslap',
-    'ridecymbalb',
-    'mutehibongo',
-    'hibongo',
-    'openhibongo',
-    'mutelobongo',
-    'lobongo',
-    'openlobongo',
-    'mutehiconga',
-    'muteloconga',
-    'openhiconga',
-    'hiconga',
-    'openloconga',
-    'loconga',
-    'hitimbale',
-    'lotimbale',
-    'hiagogo',
-    'loagogo',
-    'cabasa',
-    'maracas',
-    'shortwhistle',
-    'longwhistle',
-    'shortguiro',
-    'longguiro',
-    'guiro',
-    'claves',
-    'hiwoodblock',
-    'lowoodblock',
-    'mutecuica',
-    'opencuica',
-    'mutetriangle',
-    'triangle',
-    'opentriangle',
-    'oneup',
-    'twoup',
-    'threeup',
-    'fourup',
-    'fiveup',
-    'onedown',
-    'twodown',
-    'threedown',
-    'fourdown',
-    'fivedown',
-    'bda',
-    'bd',
-    'ssh',
-    'ss',
-    'ssl',
-    'sna',
-    'sn',
-    'hc',
-    'sne',
-    'tomfl',
-    'hhc',
-    'hh',
-    'tomfh',
-    'hhp',
-    'toml',
-    'hho',
-    'hhho',
-    'tomml',
-    'tommh',
-    'cymca',
-    'cymc',
-    'tomh',
-    'cymra',
-    'cymr',
-    'cymch',
-    'rb',
-    'tamb',
-    'cyms',
-    'cb',
-    'cymcb',
-    'vibs',
-    'cymrb',
-    'bohm',
-    'boh',
-    'boho',
-    'bolm',
-    'bol',
-    'bolo',
-    'cghm',
-    'cglm',
-    'cgho',
-    'cgh',
-    'cglo',
-    'cgl',
-    'timh',
-    'timl',
-    'agh',
-    'agl',
-    'cab',
-    'mar',
-    'whs',
-    'whl',
-    'guis',
-    'guil',
-    'gui',
-    'cl',
-    'wbh',
-    'wbl',
-    'cuim',
-    'cuio',
-    'trim',
-    'tri',
-    'trio',
-    'tt',
-    'ua',
-    'ub',
-    'uc',
-    'ud',
-    'ue',
-    'da',
-    'db',
-    'dc',
-    'dd',
-    'de',
-)
-drum_pitch_names_set = frozenset(drum_pitch_names)
+
+drum_pitch_longnames = {
+    'acousticbassdrum': (-3, 6, NATURAL),
+    'bassdrum': (-2, 0, NATURAL),
+    'hisidestick': (-3, 6, DOUBLE_SHARP),
+    'sidestick': (-2, 0, SHARP),
+    'losidestick': (-2, 1, FLAT),
+    'acousticsnare': (-2, 1, NATURAL),
+    'snare': (-2, 2, DOUBLE_FLAT),
+    'handclap': (-2, 1, SHARP),
+    'electricsnare': (-2, 2, NATURAL),
+    'lowfloortom': (-2, 3, NATURAL),
+    'closedhihat': (-2, 3, SHARP),
+    'hihat': (-2, 4, FLAT),
+    'highfloortom': (-2, 4, NATURAL),
+    'pedalhihat': (-2, 4, SHARP),
+    'lowtom': (-2, 5, NATURAL),
+    'openhihat': (-2, 5, SHARP),
+    'halfopenhihat': (-2, 5, SHARP),
+    'lowmidtom': (-2, 6, NATURAL),
+    'himidtom': (-1, 0, NATURAL),
+    'crashcymbala': (-1, 0, SHARP),
+    'crashcymbal': (-1, 1, FLAT),
+    'hightom': (-1, 1, NATURAL),
+    'ridecymbala': (-1, 1, SHARP),
+    'ridecymbal': (-1, 2, FLAT),
+    'chinesecymbal': (-1, 2, NATURAL),
+    'ridebell': (-1, 3, NATURAL),
+    'tambourine': (-1, 3, SHARP),
+    'splashcymbal': (-1, 4, NATURAL),
+    'cowbell': (-1, 4, SHARP),
+    'crashcymbalb': (-1, 5, NATURAL),
+    'vibraslap': (-1, 5, SHARP),
+    'ridecymbalb': (-1, 6, NATURAL),
+    'mutehibongo': (-1, 6, SHARP),
+    'hibongo': (0, 0, NATURAL),
+    'openhibongo': (0, 1, DOUBLE_FLAT),
+    'mutelobongo': (-1, 6, DOUBLE_SHARP),
+    'lobongo': (0, 0, SHARP),
+    'openlobongo': (0, 1, FLAT),
+    'mutehiconga': (0, 1, NATURAL),
+    'muteloconga': (0, 2, DOUBLE_FLAT),
+    'openhiconga': (0, 1, SHARP),
+    'hiconga': (0, 2, FLAT),
+    'openloconga': (0, 1, DOUBLE_SHARP),
+    'loconga': (0, 2, NATURAL),
+    'hitimbale': (0, 3, NATURAL),
+    'lotimbale': (0, 3, SHARP),
+    'hiagogo': (0, 4, NATURAL),
+    'loagogo': (0, 4, SHARP),
+    'cabasa': (0, 5, NATURAL),
+    'maracas': (0, 5, SHARP),
+    'shortwhistle': (0, 6, NATURAL),
+    'longwhistle': (1, 0, NATURAL),
+    'shortguiro': (1, 0, SHARP),
+    'longguiro': (1, 1, NATURAL),
+    'guiro': (1, 0, DOUBLE_SHARP),
+    'claves': (1, 1, SHARP),
+    'hiwoodblock': (1, 2, NATURAL),
+    'lowoodblock': (1, 3, NATURAL),
+    'mutecuica': (1, 3, SHARP),
+    'opencuica': (1, 4, NATURAL),
+    'mutetriangle': (1, 4, SHARP),
+    'triangle': (1, 4, DOUBLE_SHARP),
+    'opentriangle': (1, 5, NATURAL),
+    # 'transposing' pitches:
+    'oneup': (0, 1, NATURAL),
+    'twoup': (0, 2, NATURAL),
+    'threeup': (0, 3, NATURAL),
+    'fourup': (0, 4, NATURAL),
+    'fiveup': (0, 5, NATURAL),
+    'onedown': (-1, 6, NATURAL),
+    'twodown': (-1, 5, NATURAL),
+    'threedown': (-1, 4, NATURAL),
+    'fourdown': (-1, 3, NATURAL),
+    'fivedown': (-1, 2, NATURAL),
+}
+
+drum_pitch_shortnames = {
+    'bda': 'acousticbassdrum',
+    'bd': 'bassdrum',
+    'ssh': 'hisidestick',
+    'ss': 'sidestick',
+    'ssl': 'losidestick',
+    'sna': 'acousticsnare',
+    'sn': 'snare',
+    'hc': 'handclap',
+    'sne': 'electricsnare',
+    'tomfl': 'lowfloortom',
+    'hhc': 'closedhihat',
+    'hh': 'hihat',
+    'tomfh': 'highfloortom',
+    'hhp': 'pedalhihat',
+    'toml': 'lowtom',
+    'hho': 'openhihat',
+    'hhho': 'halfopenhihat',
+    'tomml': 'lowmidtom',
+    'tommh': 'himidtom',
+    'cymca': 'crashcymbala',
+    'cymc': 'crashcymbal',
+    'tomh': 'hightom',
+    'cymra': 'ridecymbala',
+    'cymr': 'ridecymbal',
+    'cymch': 'chinesecymbal',
+    'rb': 'ridebell',
+    'tamb': 'tambourine',
+    'cyms': 'splashcymbal',
+    'cb': 'cowbell',
+    'cymcb': 'crashcymbalb',
+    'vibs': 'vibraslap',
+    'cymrb': 'ridecymbalb',
+    'bohm': 'mutehibongo',
+    'boh': 'hibongo',
+    'boho': 'openhibongo',
+    'bolm': 'mutelobongo',
+    'bol': 'lobongo',
+    'bolo': 'openlobongo',
+    'cghm': 'mutehiconga',
+    'cglm': 'muteloconga',
+    'cgho': 'openhiconga',
+    'cgh': 'hiconga',
+    'cglo': 'openloconga',
+    'cgl': 'loconga',
+    'timh': 'hitimbale',
+    'timl': 'lotimbale',
+    'agh': 'hiagogo',
+    'agl': 'loagogo',
+    'cab': 'cabasa',
+    'mar': 'maracas',
+    'whs': 'shortwhistle',
+    'whl': 'longwhistle',
+    'guis': 'shortguiro',
+    'guil': 'longguiro',
+    'gui': 'guiro',
+    'cl': 'claves',
+    'wbh': 'hiwoodblock',
+    'wbl': 'lowoodblock',
+    'cuim': 'mutecuica',
+    'cuio': 'opencuica',
+    'trim': 'mutetriangle',
+    'tri': 'triangle',
+    'trio': 'opentriangle',
+    'tt': 'tamtam',
+    'ua': 'oneup',
+    'ub': 'twoup',
+    'uc': 'threeup',
+    'ud': 'fourup',
+    'ue': 'fiveup',
+    'da': 'onedown',
+    'db': 'twodown',
+    'dc': 'threedown',
+    'dd': 'fourdown',
+    'de': 'fivedown',
+}
+
+
+drum_pitch_names = dict(drum_pitch_longnames)
+drum_pitch_names.update((short_name, drum_pitch_longnames.get(long_name))
+    for short_name, long_name in drum_pitch_shortnames.items())
+
+
+rests = tuple("Rrs")
 
 
 durations = (
     '\\maxima', '\\longa', '\\breve',
     '1', '2', '4', '8', '16', '32', '64', '128', '256', '512', '1024', '2048',
 )
+
+
+# http://lilypond.org/doc/latest/Documentation/usage/music-fragment-options
+lilypond_book_options = (
+  "staffsize",
+  "ragged-right",
+  "noragged-right",
+  "line-width",
+  "papersize",
+  "paper-width",
+  "paper-height",
+  "notime",
+  "fragment",
+  "nofragment",
+  "inline",
+  "indent",
+  "noindent",
+  "quote",
+  "exampleindent",
+  "relative",
+  "verbatim",
+  "texidoc",
+  "doctitle",
+  "nogettext",
+  "printfilename",
+)
+
+lilypond_book_units = (
+  "cm",
+  "mm",
+  "in",
+  "pt",
+)
