@@ -151,6 +151,27 @@ class Items(list):
         if lastindex > -1:
             yield result
 
+    def peek(self, index, *values):
+        """Return True if the items from ``index`` compare equal with the
+        ``values``.
+
+        For tokens, the value is their action; for :class:`Item` instances
+        their name. Negative indices are allowed.
+
+        .. versionadded:: 0.27.0
+        """
+        if index < 0:
+            index += len(self)
+            if index < 0:
+                return False
+        if index + len(values) > len(self):
+            return False
+        for i, v in enumerate(values, index):
+            t = self[i]
+            if (t.action if t.is_token else t.name) != v:
+                return False
+        return True
+
 
 class Transform:
     """This is the base class for a transform class.
@@ -183,6 +204,13 @@ class Transformer(parce.util.Observable):
         with the tree as argument
 
     """
+
+    #: This format string creates the name to look for when searching a suitable
+    #: Transform class in a Language module space (see :meth:`find_transform`).
+    #:
+    #: .. versionadded:: 0.27.0
+    transform_name_template = "{}Transform"
+
     def __init__(self):
         super().__init__()
         self._transforms = parce.util.caching_dict(self.find_transform)
@@ -314,8 +342,8 @@ class Transformer(parce.util.Observable):
             self._cache[tree] = result
             self.emit("updated", tree, result)
         del self._interrupt[tree]
-        self.emit("finished", tree)
         yield "done"
+        self.emit("finished", tree)
 
     def interrupt(self, tree):
         """Tell the Transformer to stop transforming the specified tree."""
@@ -383,8 +411,7 @@ class Transformer(parce.util.Observable):
         """Add a Transform instance for the specified language."""
         self._transforms[language] = transform
 
-    @staticmethod
-    def find_transform(language):
+    def find_transform(self, language):
         """If no Transform was added, try to find a predefined one.
 
         This is done by looking for a Transform subclass in the language's
@@ -392,45 +419,17 @@ class Transformer(parce.util.Observable):
         So for a language class named "Css", this method tries to find a
         Transform in the same module with the name "CssTransform".
 
+        This naming scheme can be modified by setting the
+        :attr:`transform_name_template` attribute.
+
         """
         module = sys.modules[language.__module__]
-        tfname = language.__name__ + "Transform"
+        tfname = self.transform_name_template.format(language.__name__)
         tf = getattr(module, tfname, None)
         if isinstance(tf, type) and issubclass(tf, Transform):
             return tf()
 
 
-class BackgroundTransformer(Transformer):
-    """A Transformer that does its job in a background thread."""
-    def __init__(self):
-        super().__init__()
-        self._jobs = {}
-
-    def build(self, tree):
-        """Reimplemented to build the transformation in a background thread.
-
-        This method starts a background thread performing the transformation
-        and returns immediately.
-
-        """
-        def job():
-            for stage in self.process(tree):
-                pass
-            del self._jobs[tree]
-        job = self._jobs[tree] = threading.Thread(target=job)
-        job.start()
-
-    def wait(self, tree):
-        """Wait for completion of the transformation of the tree if busy."""
-        job = self._jobs.get(tree)
-        if job:
-            job.wait()
-
-    def busy(self, tree):
-        """Return True if a job transforming the tree is busy."""
-        return tree in self._jobs
-
-
 def transform_tree(tree, transform=None):
     """Convenience function that transforms tree using Transform.
 
