@@ -23,37 +23,11 @@ import re
 import reprlib
 
 import parce
-from .lexicon import LexiconDescriptor, Lexicon
+from . import introspect
+from .lexicon import Lexicon
 from .ruleitem import variations, a_number
 
 
-def validate_language(lang):
-    """Validate all lexicons in this language.
-
-    Errors and warnings are printed to stdout. If there are errors,
-    this function returns False, otherwise True.
-
-    """
-
-    lexicons = []
-    for key, value in lang.__dict__.items():
-        if isinstance(value, LexiconDescriptor):
-            lexicons.append(getattr(lang, key))
-
-    errors = set()
-    warnings = set()
-    for lexicon in lexicons:
-        v = LexiconValidator(lexicon)
-        v.validate()
-        errors.update(v.errors)
-        warnings.update(v.warnings)
-    for warning in warnings:
-        print(warning)
-    for error in errors:
-        print(error)
-    return not errors
-
-
 class LexiconValidator:
 
     def __init__(self, lexicon):
@@ -198,3 +172,40 @@ class LexiconValidator:
             else:
                 break
 
+
+def validate_language(lang):
+    """Validate all lexicons in this language.
+
+    Errors and warnings are printed to stdout. If there are errors,
+    this function returns False, otherwise True.
+
+    """
+    errors = set()
+    warnings = set()
+    for lexicon in introspect.lexicons(lang):
+        v = LexiconValidator(lexicon)
+        v.validate()
+        errors.update(v.errors)
+        warnings.update(v.warnings)
+    for warning in warnings:
+        print(warning)
+    for error in errors:
+        print(error)
+    return not errors
+
+
+def validate_transform(transform, language):
+    """Check whether the Transform has a method for every lexicon.
+
+    Prints the missing names to the screen.
+
+    """
+    ok = True
+    for lexicon in introspect.lexicons(language):
+        if not hasattr(transform, lexicon.name):
+            tf = "{}.{}".format(transform.__class__.__module__, transform.__class__.__name__)
+            print("Missing transform method in {tf} for lexicon: {name}".format(tf=tf, name=lexicon.fullname))
+            ok = False
+    return ok
+
+
